#!/usr/bin/perl -w

# This is the perl equivalent of benc.php. It works exactly the same.
# Where php wants flat arrays or associative arrays, in perl you use
# array references and hash references, respectively.
#
# Examples:
#
# benc( { type => "string", value => "spam" } )		returns "4:spam"
#
# bdec("d4:spaml11:spiced pork3:hamee")
#	returns the hash reference:
#
# HASH(0x82384c8)
#    'string' => 'd4:spaml11:spiced pork3:hamee'
#    'strlen' => 29
#    'type' => 'dictionary'
#    'value' => HASH(0x822bdf4)
#       'spam' => HASH(0x8238534)
#          'string' => 'l11:spiced pork3:hame'
#          'strlen' => 21
#          'type' => 'list'
#          'value' => ARRAY(0x8238528)
#             0  HASH(0x825ba38)
#                'string' => '11:spiced pork'
#                'strlen' => 14
#                'type' => 'string'
#                'value' => 'spiced pork'
#             1  HASH(0x825ba80)
#                'string' => '3:ham'
#                'strlen' => 5
#                'type' => 'string'
#                'value' => 'ham'

use strict;

sub benc($) {
	my ($obj) = @_;
	return if (!ref($obj) || ref($obj) ne "HASH" || !$obj->{type} || !$obj->{value});
	my $c = $obj->{value};
	my $t = $obj->{type};
	return benc_str($c) if ($t eq "string");
	return benc_int($c) if ($t eq "integer");
	return benc_list($c) if ($t eq "list");
	return benc_dict($c) if ($t eq "dictionary");
	return;
}

sub benc_str($) {
	my ($s) = @_;
	return length($s) . ":$s";
}

sub benc_int($) {
	my ($i) = @_;
	return "i" . $i . "e";
}

sub benc_list($) {
	my ($a) = @_;
	return "l" . join("", (map {benc($_)} @$a)) . "e";
}

sub benc_dict($) {
	my ($d) = @_;
	return "d" . join("", (map {benc_str($_) . benc($d->{$_})} sort(keys(%$d)))) . "e";
}

sub bdec_file($$) {
	my ($f, $ms) = @_;
	my $fd;
	open($fd, "< $f") or return;
	my $e;
	read($fd, $e, $ms);
	close($fd);
	return bdec($e);
}

sub bdec($) {
	my ($s) = @_;

	if ($s =~ /^(\d+):/) {
		my $l = $1;
		my $pl = length($l) + 1;
		my $v = substr($s, $pl, $l);
		my $ss = substr($s, 0, $pl + $l);
		return if (length($v) != $l);
		return {type => "string", value => $v, strlen => length($ss), string => $ss};
	}
	if ($s =~ /^i(\d+)e/) {
		my $v = $1;
		my $ss = "i" . $v . "e";
		return if ($v eq "-0");
		return if ($v =~ /^0./);
		return {type => "integer", value => $v, strlen => length($ss), string => $ss};
	}
	return bdec_list($s) if ($s =~ /^l/);
	return bdec_dict($s) if ($s =~ /^d/);
	return;
}

sub bdec_list($) {
	my ($s) = @_;

	return if ($s !~ /^l/);
	my $sl = length($s);
	my $i = 1;
	my $v = [];
	my $ss = "l";
	for (;;) {
		return if ($i >= $sl);
		last if (substr($s, $i, 1) eq "e");
		my $ret = bdec(substr($s, $i));
		return if (!defined($ret) || ref($ret) ne "HASH");
		push(@$v, $ret);
		$i += $ret->{strlen};
		$ss .= $ret->{string};
	}
	$ss .= "e";
	return {type => "list", value => $v, strlen => length($ss), string => $ss};
}

sub bdec_dict($) {
	my ($s) = @_;

	return if ($s !~ /^d/);
	my $sl = length($s);
	my $i = 1;
	my $v = {};
	my $ss = "d";
	for (;;) {
		return if ($i >= $sl);
		last if (substr($s, $i, 1) eq "e");
		my $ret = bdec(substr($s, $i));
		return if (!defined($ret) || ref($ret) ne "HASH" || $ret->{type} ne "string");
		my $k = $ret->{value};
		$i += $ret->{strlen};
		$ss .= $ret->{string};
		return if ($i >= $sl);
		$ret = bdec(substr($s, $i));
		return if (!defined($ret) || ref($ret) ne "HASH");
		$v->{$k} = $ret;
		$i += $ret->{strlen};
		$ss .= $ret->{string};
	}
	$ss .= "e";
	return {type => "dictionary", value => $v, strlen => length($ss), string => $ss};
}
